@@ -1,13 +1,12 @@
 # This file is part of Checkbox.
 #
-# Copyright 2013 Canonical Ltd.
+# Copyright 2012-2014 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
 # Checkbox is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 3,
 # as published by the Free Software Foundation.
-
 #
 # Checkbox is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,7 +15,6 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
-
 """
 :mod:`plainbox.impl.clitools` -- support code for command line utilities
 ========================================================================
@@ -41,6 +39,8 @@ from plainbox.i18n import gettext as _
 from plainbox.i18n import textdomain
 from plainbox.impl._argparse import LegacyHelpFormatter
 from plainbox.impl.logging import adjust_logging
+from plainbox.impl.secure.plugins import IPlugInCollection
+from plainbox.impl.secure.plugins import now
 
 
 logger = logging.getLogger("plainbox.clitools")
@@ -273,8 +273,19 @@ class ToolBase(metaclass=abc.ABCMeta):
         """
         Initialize all the variables, real stuff happens in main()
         """
+        self._setup_logging_from_environment()
         self._early_parser = None  # set in _early_init()
         self._parser = None  # set in main()
+        logger.debug(_("Constructed %r"), self)
+
+    def _setup_logging_from_environment(self):
+        if not os.getenv("PLAINBOX_DEBUG", ""):
+            return
+        adjust_logging(
+            level=os.getenv("PLAINBOX_LOG_LEVEL", "DEBUG"),
+            trace_list=os.getenv("PLAINBOX_TRACE", "").split(","),
+            debug_console=os.getenv("PLAINBOX_DEBUG", "") == "console")
+        logger.debug(_("Activated early logging via environment variables"))
 
     def main(self, argv=None):
         """
@@ -286,21 +297,30 @@ class ToolBase(metaclass=abc.ABCMeta):
         # get to the point when we do something useful and setup
         # all the exception handlers).
         try:
+            logger.debug(_("Tool initialization (early mode)"))
             self.early_init()
+            logger.debug(_("Parsing command line arguments (early mode)"))
             early_ns = self._early_parser.parse_args(argv)
+            logger.debug(
+                _("Command line parsed to (early mode): %r"), early_ns)
+            logger.debug(_("Tool initialization (late mode)"))
             self.late_init(early_ns)
             # Construct the full command line argument parser
-            self._parser = self.construct_parser()
-            logger.debug(_("parsed early namespace: %s"), early_ns)
+            logger.debug(_("Parser construction"))
+            self._parser = self.construct_parser(early_ns)
             # parse the full command line arguments, this is also where we
             # do argcomplete-dictated exit if bash shell completion
             # is requested
+            logger.debug(_("Parsing command line arguments"))
             ns = self._parser.parse_args(argv)
-            logger.debug(_("parsed full namespace: %s"), ns)
+            logger.debug(_("Command line parsed to: %r"), ns)
+            logger.debug(_("Tool initialization (final steps)"))
             self.final_init(ns)
+            logger.debug(_("Tool initialization complete"))
         except KeyboardInterrupt:
             pass
         else:
+            logger.debug(_("Dispatching command..."))
             return self.dispatch_and_catch_exceptions(ns)
 
     @classmethod
@@ -331,10 +351,21 @@ class ToolBase(metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def add_subcommands(self, subparsers):
+    def add_subcommands(self, subparsers, early_ns):
         """
         Add top-level subcommands to the argument parser.
 
+        :param subparsers:
+            The argparse subparsers object. Use it to register additional
+            command line syntax parsers and to add your commands there.
+        :param early_ns:
+            A namespace from parsing by the special early parser.  The early
+            parser may be used to quickly guess the command that needs to be
+            loaded, despite not really being able to parse everything the full
+            parser can. Using this as a hint one can optimize the command
+            loading process to skip loading commands that would not be
+            executed.
+
         This can be overridden by subclasses to use a different set of
         top-level subcommands.
         """
@@ -429,13 +460,13 @@ class ToolBase(metaclass=abc.ABCMeta):
             help=_("show program's version number and exit"))
         return parser
 
-    def construct_parser(self):
+    def construct_parser(self, early_ns=None):
         parser = self.create_parser_object()
         # Add all the things really parsed by the early parser so that it
         # shows up in --help and bash tab completion.
         self.add_early_parser_arguments(parser)
         subparsers = parser.add_subparsers()
-        self.add_subcommands(subparsers)
+        self.add_subcommands(subparsers, early_ns)
         self.enable_argcomplete_if_possible(parser)
         return parser
 
@@ -572,6 +603,153 @@ class ToolBase(metaclass=abc.ABCMeta):
                 return 1
 
 
+class LazyLoadingToolMixIn(metaclass=abc.ABCMeta):
+    """
+    Mix-in class for ToolBase that improves responsiveness by loading
+    subcommands lazily on demand and using some heuristic that works well in
+    the common case of running one command.
+
+    Unlike the original, this implementation uses a custom version of
+    add_subcommands() which uses the ``early_ns`` argument as a hint to not
+    load or register commands that are not going to be needed.
+
+    In practice ``tool --help`` doesn't benefit much but ``tool <cmd>`` can now
+    be much, much faster (and more responsive) as it only loads that one
+    command.
+
+    Concrete subclasses must implement the :meth:`get_command_collection()`
+    method which must return a IPlugInCollection (ideally the
+    LazyPlugInCollection that contains extra optimizations for low-cost key
+    enumeration and one-at-a-time value loading).
+    """
+
+    @abc.abstractmethod
+    def get_command_collection(self) -> IPlugInCollection:
+        """
+        Get a (lazy) collection of all subcommands.
+
+        This method returns a IPlugInCollection that maps command name to
+        CommandBase subclass, such as :class:`PlainBoxCommand`.
+
+        The name of each plug in object **must** match the command name.
+        """
+
+    def add_subcommands(
+        self,
+        subparsers: argparse._SubParsersAction,
+        early_ns: "Maybe[argparse.Namespace]"=None,
+    ) -> None:
+        """
+        Add top-level subcommands to the argument parser.
+
+        :param subparsers:
+            A part of argparse that can be used to create additional parsers
+            for specific subcommands.
+        :param early_ns:
+            (optional) An argparse namespace from earlier parsing. If it is not
+            None, it must have the ``rest`` attribute which is used as a list
+            of hints.
+
+        .. note::
+            This method is customized by LazyLoadingToolMixIn and should not be
+            overriden directly. To register your commands use
+            :meth:`get_command_collection()`
+        """
+        if early_ns is not None:
+            self.add_subcommands_with_hints(subparsers, early_ns.rest)
+        else:
+            self.add_subcommands_without_hints(
+                subparsers, self.get_command_collection())
+
+    def add_subcommands_with_hints(
+        self, subparsers: argparse._SubParsersAction,
+        hint_list: "List[str]"
+    ) -> None:
+        """
+        Add top-level subcommands to the argument parser, using a list of
+        hints.
+
+        :param subparsers:
+            A part of argparse that can be used to create additional parsers
+            for specific subcommands.
+        :param hint_list:
+            A list of strings that should be used as hints.
+
+        This method tries to optimize the time needed to register and setup all
+        of the subcommands by looking at a list of hints in search for the
+        (likely) command that will be executed.
+
+        Things that look like options are ignored. The first element of
+        ``hint_list`` that matches a known command name, as provided by
+        meth:`get_command_collection()`, is used as a sign that that command
+        will be executed and all other commands don't have to be loaded or
+        initialized. If no hints are found (e.g. when running ``tool --help``)
+        the slower fallback mode is used and all subcommands are added.
+
+        .. note::
+            This method is customized by LazyLoadingToolMixIn and should not be
+            overriden directly. To register your commands use
+            :meth:`get_command_collection()`
+        """
+        logger.debug(
+            _("Trying to load exactly the right command: %r"), hint_list)
+        command_collection = self.get_command_collection()
+        for hint in hint_list:
+            # Skip all the things that look like additional options
+            if hint.startswith('-'):
+                continue
+            # Break on the first hint that we can load
+            try:
+                plugin = command_collection.get_by_name(hint)
+            except KeyError:
+                continue
+            else:
+                command = plugin.plugin_object
+                logger.debug("Registering single command %r", command)
+                start = now()
+                command.register_parser(subparsers)
+                logger.debug(_("Cost of registering guessed command: %f"),
+                             now() - start)
+                break
+        else:
+            logger.debug("Falling back to loading all commands")
+            self.add_subcommands_without_hints(subparsers, command_collection)
+
+    def add_subcommands_without_hints(
+        self, subparsers: argparse._SubParsersAction,
+        command_collection: IPlugInCollection,
+    ) -> None:
+        """
+        Add top-level subcommands to the argument parser (fallback mode)
+
+        :param subparsers:
+            A part of argparse that can be used to create additional parsers
+            for specific subcommands.
+        :param command_collection:
+            A collection of commands that was obtaioned from
+            :meth:`get_command_collection()` earlier.
+
+        This method is called when hint-based optimization cannot be used and
+        all commands need to be loaded and initialized.
+
+        .. note::
+            This method is customized by LazyLoadingToolMixIn and should not be
+            overriden directly. To register your commands use
+            :meth:`get_command_collection()`
+        """
+        command_collection.load()
+        logger.debug(
+            _("Cost of loading all top-level commands: %f"),
+            command_collection.get_total_time())
+        start = now()
+        for command in command_collection.get_all_plugin_objects():
+            logger.debug("Registering command %r", command)
+            command.register_parser(subparsers)
+        logger.debug(
+            _("Cost of registering all top-level commands: %f"),
+            now() - start)
+
+
 class SingleCommandToolMixIn:
     """
     Mix-in class for ToolBase to implement single-command dispatch.
@@ -589,14 +767,14 @@ class SingleCommandToolMixIn:
         :meth:`CommandBase.register_arguments()` method.
         """
 
-    def add_subcommands(self, subparsers):
+    def add_subcommands(self, subparsers, early_ns):
         """
         Overridden version of add_subcommands()
 
         This method does nothing. It is here because ToolBase requires it.
         """
 
-    def construct_parser(self):
+    def construct_parser(self, early_ns=None):
         """
         Overridden version of construct_parser()
 
