@@ -62,10 +62,16 @@ class SessionCommand(PlainBoxCommand):
         parser = self.add_subcommand(subparsers)
         parser.prog = 'plainbox session'
         parser.set_defaults(default_session_cmd='list')
+        # Duplicate the default value of --only-ids This is only used when
+        # we use the default command aka when 'plainbox session' runs.
+        parser.set_defaults(only_ids=False)
         session_subparsers = parser.add_subparsers(
             title=_('available session subcommands'))
         list_parser = session_subparsers.add_parser(
             'list', help=_('list available sessions'))
+        list_parser.add_argument(
+            '--only-ids', help=_('print one id per line only'),
+            action='store_true', default=False)
         list_parser.set_defaults(session_cmd='list')
         remove_parser = session_subparsers.add_parser(
             'remove', help=_('remove one more more sessions'))
@@ -99,6 +105,9 @@ class SessionCommand(PlainBoxCommand):
         export_parser.add_argument(
             'session_id', metavar=_('SESSION-ID'),
             help=_('Identifier of the session to export'))
+        export_parser.add_argument(
+            '--flag', action='append', metavar=_("FLAG"),
+            help=_("pass this resume flag to the session resume code"))
         export_parser.set_defaults(session_cmd='export')
         group = export_parser.add_argument_group(_("output options"))
         group.add_argument(
