@@ -40,18 +40,19 @@ class TestScriptCommand(TestCase):
     def setUp(self):
         self.parser = argparse.ArgumentParser(prog='test')
         self.subparsers = self.parser.add_subparsers()
-        self.provider_loader = lambda:[mock.Mock()]
-        self.config = mock.Mock()
+        self.provider_loader = lambda: [mock.Mock()]
+        self.config_loader = lambda: mock.Mock()
         self.ns = mock.Mock()
 
     def test_init(self):
-        script_cmd = ScriptCommand(self.provider_loader, self.config)
+        script_cmd = ScriptCommand(self.provider_loader, self.config_loader)
         self.assertIs(script_cmd.provider_loader, self.provider_loader)
-        self.assertIs(script_cmd.config, self.config)
+        self.assertIs(script_cmd.config_loader, self.config_loader)
 
     def test_register_parser(self):
-        ScriptCommand(self.provider_loader, self.config).register_parser(
-            self.subparsers)
+        ScriptCommand(
+            self.provider_loader, self.config_loader
+        ).register_parser(self.subparsers)
         with TestIO() as io:
             self.parser.print_help()
         self.assertIn("script    run a command from a job", io.stdout)
@@ -74,13 +75,15 @@ class TestScriptCommand(TestCase):
     @mock.patch("plainbox.impl.commands.inv_script.ScriptInvocation")
     def test_invoked(self, patched_ScriptInvocation):
         retval = ScriptCommand(
-            self.provider_loader, self.config).invoked(self.ns)
+            self.provider_loader, self.config_loader
+        ).invoked(self.ns)
         patched_ScriptInvocation.assert_called_once_with(
-            self.provider_loader, self.config, self.ns.job_id)
+            self.provider_loader, self.config_loader, self.ns.job_id)
         self.assertEqual(
             retval, patched_ScriptInvocation(
-                self.provider_loader, self.config,
-                self.ns.job_id).run.return_value)
+                self.provider_loader, self.config_loader,
+                self.ns.job_id
+            ).run.return_value)
 
 
 class ScriptInvocationTests(TestCase):
@@ -89,7 +92,7 @@ class ScriptInvocationTests(TestCase):
 
     def setUp(self):
         self.provider_loader = mock.Mock()
-        self.config = PlainBoxConfig()
+        self.config_loader = lambda: PlainBoxConfig()
         self.job_id = mock.Mock()
 
     def assertCommandOutput(self, actual, expected):
@@ -97,14 +100,15 @@ class ScriptInvocationTests(TestCase):
 
     def test_init(self):
         script_inv = ScriptInvocation(
-            self.provider_loader, self.config, self.job_id)
+            self.provider_loader, self.config_loader, self.job_id)
         self.assertIs(script_inv.provider_loader, self.provider_loader)
-        self.assertIs(script_inv.config, self.config)
+        self.assertIs(script_inv.config_loader, self.config_loader)
         self.assertIs(script_inv.job_id, self.job_id)
 
     def test_run_no_such_job(self):
         provider_loader = lambda: [DummyProvider1()]
-        script_inv = ScriptInvocation(provider_loader, self.config, self.JOB_ID)
+        script_inv = ScriptInvocation(
+            provider_loader, self.config_loader, self.JOB_ID)
         with TestIO() as io:
             retval = script_inv.run()
         self.assertCommandOutput(
@@ -117,7 +121,8 @@ class ScriptInvocationTests(TestCase):
 
     def test_run_job_without_command(self):
         provider_loader = lambda: [DummyProvider1([make_job(self.JOB_PARTIAL_ID)])]
-        script_inv = ScriptInvocation(provider_loader, self.config, self.JOB_ID)
+        script_inv = ScriptInvocation(
+            provider_loader, self.config_loader, self.JOB_ID)
         with TestIO() as io:
             retval = script_inv.run()
         self.assertCommandOutput(
@@ -131,7 +136,8 @@ class ScriptInvocationTests(TestCase):
     def test_job_with_command(self, mock_check_output):
         provider_loader = lambda: [DummyProvider1([
             make_job(self.JOB_PARTIAL_ID, command='echo ok')])]
-        script_inv = ScriptInvocation(provider_loader, self.config, self.JOB_ID)
+        script_inv = ScriptInvocation(
+            provider_loader, self.config_loader, self.JOB_ID)
         with TestIO() as io:
             retval = script_inv.run()
         self.assertCommandOutput(
@@ -147,7 +153,8 @@ class ScriptInvocationTests(TestCase):
     def test_job_with_command_making_files(self, mock_check_output):
         provider_loader = lambda: [DummyProvider1([
             make_job(self.JOB_PARTIAL_ID, command='echo ok > file')])]
-        script_inv = ScriptInvocation(provider_loader, self.config, self.JOB_ID)
+        script_inv = ScriptInvocation(
+            provider_loader, self.config_loader, self.JOB_ID)
         with TestIO() as io:
             retval = script_inv.run()
         self.maxDiff = None
@@ -165,7 +172,8 @@ class ScriptInvocationTests(TestCase):
     def test_job_with_command_making_directories(self, mock_check_output):
         provider_loader = lambda: [DummyProvider1([
             make_job(self.JOB_PARTIAL_ID, command='mkdir dir')])]
-        script_inv = ScriptInvocation(provider_loader, self.config, self.JOB_ID)
+        script_inv = ScriptInvocation(
+            provider_loader, self.config_loader, self.JOB_ID)
         with TestIO() as io:
             retval = script_inv.run()
         self.maxDiff = None
