@@ -136,6 +136,15 @@ class CommandOutputWriter(extcmd.DelegateBase):
         self.stdout.close()
         self.stderr.close()
 
+    def on_abnormal_end(self, signal_num):
+        """
+        Internal method of extcmd.DelegateBase
+
+        Called when a command abnormally finishes running
+        """
+        self.stdout.close()
+        self.stderr.close()
+
     def on_line(self, stream_name, line):
         """
         Internal method of extcmd.DelegateBase
@@ -213,6 +222,18 @@ class JobRunnerUIDelegate(extcmd.DelegateBase):
         if self.ui is not None:
             self.ui.finished_executing_program(returncode)
 
+    def on_abnormal_end(self, signal_num):
+        """
+        Internal method of extcmd.DelegateBase
+
+        Called when a command abnormally finishes running
+
+        The negated signal number is used as the exit code of the program and
+        fed into the UI (if any)
+        """
+        if self.ui is not None:
+            self.ui.finished_executing_program(-signal_num)
+
     def on_line(self, stream_name, line):
         """
         Internal method of extcmd.DelegateBase
@@ -762,7 +783,8 @@ class JobRunner(IJobRunner):
         delegate, io_log_gen = self._prepare_io_handling(job, config)
         # Create a subprocess.Popen() like object that uses the delegate
         # system to observe all IO as it occurs in real time.
-        extcmd_popen = extcmd.ExternalCommandWithDelegate(delegate)
+        delegate_cls = self._get_delegate_cls(config)
+        extcmd_popen = delegate_cls(delegate)
         # Stream all IOLogRecord entries to disk
         record_path = os.path.join(
             self._jobs_io_log_dir, "{}.record.gz".format(
@@ -856,3 +878,14 @@ class JobRunner(IJobRunner):
             logger.warning(
                 _("Please store desired files in $PLAINBOX_SESSION_SHARE and"
                   " use regular temporary files for everything else"))
+
+    def _get_delegate_cls(self, config):
+        if (sys.version_info[0:2] >= (3, 4) and sys.platform == 'linux'
+                and config.extcmd == "glibc"):
+            logger.debug("Using glibc-based command runner")
+            from plainbox.vendor.extcmd.glibc import (
+                GlibcExternalCommandWithDelegate)
+            return GlibcExternalCommandWithDelegate
+        else:
+            logger.debug("Using classic thread-based command runner")
+            return extcmd.ExternalCommandWithDelegate
