@@ -26,6 +26,7 @@ Test definitions for plainbox.impl.session module
 
 from unittest import TestCase
 
+from plainbox.abc import IExecutionController
 from plainbox.abc import IJobResult
 from plainbox.impl.depmgr import DependencyDuplicateError
 from plainbox.impl.depmgr import DependencyMissingError
@@ -916,3 +917,106 @@ class SessionDeviceContextTests(TestCase, SignalInterceptorMixIn):
         sig2 = self.assertSignalFired(self.ctx.state.on_unit_removed, self.job)
         sig3 = self.assertSignalFired(self.ctx.state.on_job_removed, self.job)
         self.assertSignalOrdering(sig1, sig2, sig3)
+
+    def test_execution_controller_list__computed(self):
+        """
+        Ensure that the list of execution controllers is computed correctly
+        """
+        with mock.patch.object(self.ctx, '_compute_execution_ctrl_list') as m:
+            result = self.ctx.execution_controller_list
+        self.assertIs(result, m())
+        m.assert_called_once()
+
+    def test_execution_controller_list__cached(self):
+        """
+        Ensure that the computed list of execution controllers is cached
+        """
+        self.assertNotIn(
+            SessionDeviceContext._CACHE_EXECUTION_CTRL_LIST,
+            self.ctx._shared_cache)
+        with mock.patch.object(self.ctx, '_compute_execution_ctrl_list') as m:
+            result1 = self.ctx.execution_controller_list
+            result2 = self.ctx.execution_controller_list
+        self.assertIs(result1, result2)
+        m.assert_called_once()
+        self.assertIn(
+            SessionDeviceContext._CACHE_EXECUTION_CTRL_LIST,
+            self.ctx._shared_cache)
+
+    def test_execution_controller_list__invalidated(self):
+        """
+        Ensure that the cached list of execution controllers is invalidated
+        when a new provider is added to the context
+        """
+        # Let's have a fake provider ready. We need to mock the get_units()
+        # method to let us add it to the context.
+        provider2 = mock.Mock(name='provider2', spec_set=Provider1)
+        provider2.get_units.return_value = ([], [])
+        with mock.patch.object(self.ctx, '_compute_execution_ctrl_list') as m:
+            m.side_effect = lambda: mock.Mock()
+            self.assertNotIn(
+                SessionDeviceContext._CACHE_EXECUTION_CTRL_LIST,
+                self.ctx._shared_cache)
+            result1 = self.ctx.execution_controller_list
+            self.assertIn(
+                SessionDeviceContext._CACHE_EXECUTION_CTRL_LIST,
+                self.ctx._shared_cache)
+            # Adding the second provider should invalidate the cache
+            self.ctx.add_provider(provider2)
+            self.assertNotIn(
+                SessionDeviceContext._CACHE_EXECUTION_CTRL_LIST,
+                self.ctx._shared_cache)
+            result2 = self.ctx.execution_controller_list
+            self.assertIn(
+                SessionDeviceContext._CACHE_EXECUTION_CTRL_LIST,
+                self.ctx._shared_cache)
+        # Both results are different
+        self.assertNotEqual(result1, result2)
+        # And _compute_execution_ctrl_list was called twice
+        m.assert_has_calls(((), {}), ((), {}))
+
+    def test_get_ctrl_for_job__best(self):
+        """
+        Ensure that get_ctrl_for_job() picks the best execution controller
+        out of the available choices.
+        """
+        ctrl1 = mock.Mock(name='ctrl1', spec_set=IExecutionController)
+        ctrl1.get_score.return_value = 5
+        ctrl2 = mock.Mock(name='ctrl2', spec_set=IExecutionController)
+        ctrl2.get_score.return_value = 7
+        ctrl3 = mock.Mock(name='ctrl3', spec_set=IExecutionController)
+        ctrl3.get_score.return_value = -1
+        with mock.patch.object(self.ctx, '_compute_execution_ctrl_list') as m:
+            m.return_value = [ctrl1, ctrl2, ctrl3]
+            best_ctrl = self.ctx.get_ctrl_for_job(self.job)
+        self.assertIs(best_ctrl, ctrl2)
+
+    def test_get_ctrl_for_job__tie(self):
+        """
+        Ensure that get_ctrl_for_job() pick the last, best controller,
+        as determined by the order of entries in execution_controller_list
+        """
+        ctrl1 = mock.Mock(name='ctrl1', spec_set=IExecutionController)
+        ctrl1.get_score.return_value = 1
+        ctrl2 = mock.Mock(name='ctrl2', spec_set=IExecutionController)
+        ctrl2.get_score.return_value = 1
+        with mock.patch.object(self.ctx, '_compute_execution_ctrl_list') as m:
+            m.return_value = [ctrl1, ctrl2]
+            best_ctrl = self.ctx.get_ctrl_for_job(self.job)
+        self.assertIs(best_ctrl, ctrl2)
+
+    def test_get_ctrl_for_job__no_candidates(self):
+        """
+        Ensure that get_ctrl_for_job() raises LookupError if no controllers
+        are suitable for the requested job.
+        """
+        ctrl1 = mock.Mock(name='ctrl1', spec_set=IExecutionController)
+        ctrl1.get_score.return_value = -1
+        ctrl2 = mock.Mock(name='ctrl1', spec_set=IExecutionController)
+        ctrl2.get_score.return_value = -1
+        ctrl3 = mock.Mock(name='ctrl1', spec_set=IExecutionController)
+        ctrl3.get_score.return_value = -1
+        with mock.patch.object(self.ctx, '_compute_execution_ctrl_list') as m:
+            m.return_value = [ctrl1, ctrl2, ctrl3]
+            with self.assertRaises(LookupError):
+                self.ctx.get_ctrl_for_job(self.job)
