@@ -1336,7 +1336,7 @@ class ProviderManagerTool(ToolBase):
         """
         return "{}.{}.{}".format(*version[:3])
 
-    def add_subcommands(self, subparsers):
+    def add_subcommands(self, subparsers, early_ns=None):
         """
         Add top-level subcommands to the argument parser.
         """
